/*
 * Copyright (C) 2012-2019 Japan Smartphone Security Association
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jssec.android.sharedmemory.inhouseservice.messenger;

import org.jssec.android.shared.SigPerm;
import org.jssec.android.shared.Utils;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.os.SharedMemory;
import android.system.ErrnoException;
import android.util.Log;
import android.widget.Toast;

import java.nio.ByteBuffer;

import static android.content.pm.PackageManager.PERMISSION_GRANTED;
import static android.system.OsConstants.PROT_EXEC;
import static android.system.OsConstants.PROT_READ;
import static android.system.OsConstants.PROT_WRITE;

public class SHMService extends Service {
    // 自社の Signature Permission
    private static final String MY_PERMISSION = "org.jssec.android.sharedmemory.inhouseservice.messenger.MY_PERMISSION";

    // 自社の証明書のハッシュ値
    private static String sMyCertHash = null;
    private static String myCertHash(Context context) {
        if (sMyCertHash == null) {
            if (Utils.isDebuggable(context)) {
                // debug.keystore の"androiddebugkey"の証明書ハッシュ値
                sMyCertHash = "0EFB7236 328348A9 89718BAD DF57F544 D5CCB4AE B9DB34BC 1E29DD26 F77C8255";
            } else {
                // keystore の"my company key"の証明書ハッシュ値
                sMyCertHash = "D397D343 A5CBC10F 4EDDEB7C A10062DE 5690984F 1FB9E88B D7B3A7C2 42E142CA";
            }
        }
        return sMyCertHash;
    }

    private final String TAG = "SHM";

    // クライアントへ送信するメッセージ文字列
    private final String greeting = "Hi! I send you my memory. Let's Share it!";
    // ページサイズは 4K bytes
    public static final int PAGE_SIZE = 1024 * 4;
    // 共有メモリを2つ使用する。クライアントは下の識別子を指定してどの共有メモリが欲しいかを指定する
    public static final int SHMEM1 = 0;
    public static final int SHMEM2 = 1;

    // 共有メモリのインスタンス
    // mSHMem1: クライアントへ渡すデータ用として使用する
    private SharedMemory mSHMem1 = null;
    // mSHMem をマップするためのバイトバッファ
    private ByteBuffer m1Buffer1;
    // mSHMem2: クライアントからのデータ受け取り用として使用する
    private SharedMemory mSHMem2 = null;
    // mSHMem2 をマップするためのバイトバッファ。二つの領域に分割して用いる
    private ByteBuffer m2Buffer1;
    private ByteBuffer m2Buffer2;
    // すべてのバイトバッファが正常にマップされたときに true となる
    private boolean mBufferMapped = false;


    // この例ではクライアントとは Messenger 機構を利用して通信する
    // 以下はそこで使用するメッセージ識別子
    public static final int MSG_INVALID = Integer.MIN_VALUE;
    public static final int MSG_ATTACH  = MSG_INVALID + 1; // クライアントが SHMEM1 アロケート要求
    public static final int MSG_ATTACH2 = MSG_ATTACH + 1;  // クライアントが SHMEM2 アロケート要求
    public static final int MSG_DETACH  = MSG_ATTACH2 + 1; // クライアントが SHMEM1 を使い終わった
    public static final int MSG_DETACH2 = MSG_DETACH + 1;  // クライアントが SHMEM2 を使い終わった
    public static final int MSG_REPLY1  = MSG_DETACH2 + 1; // クライアントからの返答１
    public static final int MSG_REPLY2  = MSG_REPLY1 + 1;  // クライアントからの返答 2
    public static final int MSG_END     = MSG_REPLY2 + 1;  // Service が終了を宣言

    // クライアントから受け取った Message を処理する Handler
    private class CommHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
            case MSG_ATTACH:
                Log.d(TAG, "got MSG_ATTACH");
                shareWith1(msg);
                break;
            case MSG_ATTACH2:
                Log.d(TAG, "got MSG_ATTACH2");
                shareWith2(msg);
                break;
            case MSG_DETACH:
                Log.d(TAG, "got MSG_DETACH");
                unShare(msg);
                break;
            case MSG_REPLY1:
                Log.d(TAG, "got MSG_REPLY1");
                gotReply(msg);
                break;
            case MSG_REPLY2:
                Log.d(TAG, "got MSG_REPLY2");
                gotReply2(msg);
                break;
            default:
                invalidMsg(msg);
            }
        }
    }

    private final Handler mHandler = new CommHandler();

    // クライアントからのデータを受信するときに利用する Messenger
    private final Messenger mMessenger = new Messenger(mHandler);

    // バインド時に Messenger から Binder を取得しクライアントへ渡す
    @Override
    public IBinder onBind(Intent intent) {
        // ★ポイント4★ 独自定義Signature Permission が自社アプリにより定義されていることを確認する
        if (!SigPerm.test(this, MY_PERMISSION, myCertHash(this))) {
            Toast.makeText(this, "独自定義 Signature Permission が自社アプリによって定義されていない。", Toast.LENGTH_LONG).show();
            return null;
        }

        // ★ポイント5★ 自社アプリからのIntent であっても、受信Intent の安全性を確認する
        // サンプルにつき割愛。「3.2 入力データの安全性を確認する」を参照。
        String param = intent.getStringExtra("PARAM");
        // Toast.makeText(this, String.format("パラメータ「%s」を受け取った。", param), Toast.LENGTH_LONG).show();
        Log.d(TAG, String.format("パラメータ[%s]を受け取った！", param));
        return mMessenger.getBinder();
    }
    
    // マップ構造
    // オフセットはページ境界でなければならない
    public static final int SHMEM1_BUF1_OFFSET = 0;
    public static final int SHMEM1_BUF1_LENGTH = 1024;
    public static final int SHMEM2_BUF1_OFFSET = 0;
    public static final int SHMEM2_BUF1_LENGTH = 1024;
    public static final int SHMEM2_BUF2_OFFSET = PAGE_SIZE;
    public static final int SHMEM2_BUF2_LENGTH = 128;

    // 2つの共有メモリをアロケートする
    private boolean allocateSharedMemory() {
        try {
            // クライアントへ渡すデータ用共有メモリ
            mSHMem1 = SharedMemory.create("SHM", PAGE_SIZE);
            // クライアントからのデータ受け取り用共有メモリ
            mSHMem2 = SharedMemory.create("SHM2", PAGE_SIZE * 2);
        } catch (ErrnoException e) {
            Log.e(TAG, "failed to allocate shared memory" + e.getMessage());
            return false;
        }
        return true;
    }

    // 指定の共有メモリをマップする
    private ByteBuffer mapShared(SharedMemory mem, int prot, int offset, int size) {
        ByteBuffer tBuf ;
        try {
            tBuf = mem.map(prot, offset, size);
        } catch (ErrnoException e) {
            Log.e(TAG, "could not map, prot=" + prot + ", offset=" + offset + ", length=" + size + "\n " + e.getMessage() + "err no. = " + e.errno);
            return null;
        } catch (IllegalArgumentException e){
            Log.e(TAG, "map failed: " + e.getMessage());
            return null;
        }
        Log.d(TAG, "mmap success: prot=" + prot);
        return tBuf;
    }

    // 自分用に二つの共有メモリをマップ
    private void mapMemory() {
        // mSHMem1 : 読み書きする
        m1Buffer1 = mapShared(mSHMem1, PROT_READ | PROT_WRITE | PROT_EXEC, SHMEM1_BUF1_OFFSET, SHMEM1_BUF1_LENGTH);
        // mSHMem2 : 二つの領域に分割し、それぞれを読み書きする
        m2Buffer1 = mapShared(mSHMem2, PROT_READ | PROT_WRITE, SHMEM2_BUF1_OFFSET, SHMEM2_BUF1_LENGTH);
        m2Buffer2 = mapShared(mSHMem2, PROT_READ | PROT_WRITE, SHMEM2_BUF2_OFFSET, SHMEM2_BUF2_LENGTH);

        if (m1Buffer1 != null && m2Buffer1 != null && m2Buffer2 != null) mBufferMapped = true;
    }

    // 共有メモリを解放する
    private void deAllocateSharedMemory () {
        if (mBufferMapped) {
            if (mSHMem1 != null) {
                if (m1Buffer1 != null) SharedMemory.unmap(m1Buffer1);
                m1Buffer1 = null;
                mSHMem1.close();
                mSHMem1 = null;
            }
            if (mSHMem2 != null) {
                if (m2Buffer1 != null) SharedMemory.unmap(m2Buffer1);
                if (m2Buffer2 != null) SharedMemory.unmap(m2Buffer2);
                m2Buffer1 = null;
                m2Buffer2 = null;
                mSHMem2.close();
                mSHMem2 = null;
            }
            mBufferMapped = false;
        }
    }

    @Override
    public void onCreate() {
        super.onCreate();

        // インスタンス生成のタイミングで共有メモリをアロケート
        // アロケートできたら共有メモリをマップする
        if (allocateSharedMemory()) {
            mapMemory();
        }
    }

    // SHMEM1 をクライアントへ提供
    private void shareWith1(Message msg){
        // アロケートされていなかったらなにもしない
        if (!mBufferMapped) return;

        // ★ポイント6★ クライアントに共有メモリを渡す前に SharedMemory#setProtect()によってクライアント側が可能な操作を制限する
        // mSHMem1 はクライアントに読み込みのみを許可する
        mSHMem1.setProtect(PROT_READ);

        // 上の setProtect(PROT_READ) 実行以前にマッピングが完了しているため、書き込みが可能
        // mBuffer の先頭にメッセージのサイズを入れ、続いてメッセージ文字列を追加
        m1Buffer1.putInt(greeting.length());
        m1Buffer1.put(greeting.getBytes());

        try {
            // 要求元に共有メモリオブジェクトを渡す
            Message sMsg = Message.obtain(null, SHMEM1, mSHMem1);
            msg.replyTo.send(sMsg);
        } catch (RemoteException e) {
            Log.e(TAG, "Failed to share" + e.getMessage());
        }
    }

    // SHMEM2 をクライアントへ提供
    private void shareWith2(Message msg) {
        if (!mBufferMapped) return;

        // ★ポイント6★ クライアントに共有メモリを渡す前に SharedMemory#setProtect()によってクライアント側が可能な操作を制限する
        // mSHMem2 はクライアントに書き込みを許可する
        mSHMem2.setProtect(PROT_WRITE);
        final String greeting2 = "You can write here!";
        m2Buffer1.putInt(greeting2.length());
        m2Buffer1.put(greeting2.getBytes());
        final String greeting3 = "From this point, I'll also write.";
        m2Buffer2.putInt(greeting3.length());
        m2Buffer2.put(greeting3.getBytes());
        try {
            // 要求元に共有メモリオブジェクトを渡す
            Message sMsg = Message.obtain(null, SHMEM2, mSHMem2);
            msg.replyTo.send(sMsg);
        } catch (RemoteException e){
            Log.e(TAG, "failed to share mSHMem2" + e.getMessage());
        }
    }

    // メモリの共有を止める
    private void unShare(Message msg){
        deAllocateSharedMemory();
    }

    // 不正なメッセージを受け取った
    private void invalidMsg(Message msg){
        Log.e(TAG, "Got an Invalid message: " + msg.what);
    }

    // クライアントから受け取った情報を取り出す
    // 先頭にサイズ、それに続いて文字バイト列が格納されているものとする
    private String extractReply (ByteBuffer buf){
        int len = buf.getInt();
        byte [] bytes = new byte[len];
        buf.get(bytes);
        return new String(bytes);
    }

    // この例では2種類のメッセージをクライアントから受け取る
    // こちらは m1Buffer1 にクライアントからの情報が格納されていることを想定
    private void gotReply(Message msg) {
        m1Buffer1.rewind();
        String message = extractReply(m1Buffer1);
        if (!message.equals(greeting)){
            Log.e(TAG, "my message was overwritten: " + message);
        }
    }
    // こちらは m2Buffer1 にクライアントからの情報が格納されていることを想定
    private void gotReply2(Message msg) {
        m2Buffer1.rewind();
        String message = extractReply(m2Buffer1);
        android.util.Log.d(TAG, "got a message of length " + message.length() + " from client: " + message);
        // このリプライメッセージが来た場合は、共有メモリの利用を終了することをクライアントと取り決めている
        Message eMsg = Message.obtain();
        eMsg.what = MSG_END;
        try {
            msg.replyTo.send(eMsg);
        } catch (RemoteException e){
            Log.e(TAG, "error in reply 2: " + e.getMessage());
        }
    }
}
